function [Mix, times] = mixestqbs(Mix0, varargin)
% function [Mix, mixlls] = mixestqb(Mix0, frg, ndat, niter, threshold)
% quasi-Bayes iterative estimation of normal mixture
%
%   LHS = mixestqbs(Mix0, frg, ndat, niter, threshold)
%   LHS = mixestqbs(Mix0, frg, ndat, niter) default threshold
%   LHS = mixestqbsMix0, frg, ndat)     niter = 1
%   LHS = mixestqbs(Mix0, frg)          ndat  = length of global DATA
%   LHS = mixestqbs(Mix0)               frg   = defaults('frg')
%
% LHS   : Mix or [Mix, times]
% Mix   : estimated mixture
% Mix0  : initial mixture
% mixlls: trajectory of mixll, i.e. log-likelihood of the mixture
% frg   : forgetting rate
% ndat  : size of data sample or data sample (copied to global DATA)
% niter : number of iterations
% threshold: stopping threshold
% times : stoping time individual iterations
%
% Note  : by setting global DEBUG>1:
%         in each iteration, start and end mixture is displayed
%
% Design  : P. Nedoma
% Updated : May 2004
% Project : GA CR 102/03/0049, AV CR S1075351, S1075102, DESIGNER


% Calls   : mixestim, defaults, mixflatv, mixflat
global TIME

Mix0 = mix2mix(Mix0, 21);              % convert mixture
[frg, ndat, niter, dbg, threshold] = ...          % process inputs
   inputs(varargin);

[Mix, times]  = mixestims(Mix0, frg, ndat);    % 1st estimation step
if niter == 1, return, end             % return if niter==1


[Mix0, handle] = ...                   % beginning of flattening
   mixflatv(Mix, niter, ndat, frg);

for iter = 2:niter % === over iterations ===================
    [Mix0, handle] = ...               % mixture flattening
        mixflatv(Mix, handle);
    
    [Mix, tstop] = mixestims(Mix0, frg, ndat);

    times(iter)  = tstop;
   
    if dbg>1,                          % show results
       iterplot(Mix0, Mix, iter,ndat);
    end

end % === over iterations ==================================

Mix = Mix0;                            % resulting mixture

%### inputs #######################################
function [frg, ndat, niter, dbg, threshold] = inputs(varargin);
% processing of inputs
% varargin = { frg, ndat, niter}
%
% frg   : forgetting rate
% ndat  : size of data sample or data sample (copied to global DATA)
% niter : number of iterations
% dbg   : setting of global DEBUG (for plots);
% threshold
%  mixestqbs(Mix0, frg, ndat, niter, threshold)
inp = varargin{1};
ni  = length(inp);

global DATA DEBUG
if length(DEBUG)==0, dbg = 0; else, dbg = DEBUG; end
if ni<5, x=defaults('E'); threshold = x(1); 
else threshold=inp{4}; 
end
 
if ni < 3, niter = 1; else niter = inp{3}; end
if ni < 2, ndat = size(DATA,2);
   if ndat<1, error('global DATA not specified'); end
elseif ni>=2,
   ndat = inp{2};
   [m,n] = size(ndat);
   if m*n>1, DATA = ndat; ndat = n; end
end
if ni < 1, frg = defaults('frg'); else frg = inp{1}; end
