% NEW mixinit
% initialization of mixture estimation
%
% [Mix, Mix0] = mixinit(Mix0,  frg,  ndat,  niter,  options,  belief)
% [Mix, Mix0] = mixinit(Mix0,  frg,  ndat,  niter,  options)  belief is not used
% [Mix, Mix0] = mixinit(Mix0,  frg,  ndat,  niter)  defaults are used
% [Mix, Mix0] = mixinit(Mix0,  frg,  ndat)          niter=5
% [Mix, Mix0] = mixinit(Mix0,  frg)                 size(DATA, 2)
% [Mix, Mix0] = mixinit(Mix0)                       default forgetting rate
%
% Mix    : estimated mixture, type 21
% Mix0   : initial mixture, any type internally converted to 21
% frg    : forgetting rate
% ndat   : size of data sample
%          it can be cell vector for buffered data processing
% niter  : number of iterations
% options: initialization options
%          character string or character string followed an number
% belief : cell vector of beliefs for individual channels
%
% --- options for estimation ---
% p : projection
% q : iterative quasi-Bayes mixture estimation
% b : iterative batch quasi-Bayes mixture estimation
% f : iterative mixture estimation based on forgetting branching
% Q : quasi-Bayes with stopping
% P : projection with stopping
% m : iterative quasi-Bayes with fixed variances
% n : number of iterations for iterative estimation, a number follows;
%     default is 10 iterations
% Note: if iterative estimation is not explicitly specified,
%       mixestim is used
%
% --- option for structure estimation ---
% h : number of runs for structure estimation (integer follows)
%
% --- options that modify processing ---
% c : do not make the final housekeeping,                   default: make it
% g : number of initial steps when all factors are split    default: 2
% k : number of steps when components are not merged or erased, default: 1
%
% See also: commerge, mixerase, mixest
% Design : M. Karny, P. Nedoma
% Updated: September 2002
% Project: ProDaCTools

% Updated: 28.9.02, MK, patch marked by MK
%
% ==========================================================
function [Mix, Mix0] = mixinit(varargin)
% ==========================================================
% globals
global DATA TIME DEBUG NDAT BREAKPOINT
if length(DEBUG)>0, dbg = DEBUG; else, dbg=0; end
Mix0 = varargin{1};                    % initial mixture

% ==========================================================
% Breakpoint option specified via global BREAKPOINT - if defined,
% it is a filename. To this file, intermediate are stored after
% each iteration. Thus, the computation can be restarted by
%         Mix = mixinit('filename', nextiter),
% where "filename" is the file specified by BREAKPOINT and "nexiter"
% is number of iterations to be done: it replaces options made before.
% ==========================================================
%
if ischar(Mix0)                        % load the interrupted processing
   BP=BREAKPOINT; DB = DEBUG; eval(['load ',Mix0]);
   BREAKPOINT=BP;  DEBUG=DB;  clear BP DB
   iter = iter+1;
   if nargin==2, niter = iter + varargin{2} - 1; end
   if iter>niter, iter = niter; end
%
% ==========================================================
else                                   % --- not a breakpoint ---
% ==========================================================
% decompose inputs, set defaults etc. done by function "inputs"
[Mix0, frg, ndat, niter, a, c, d, g, k, ...
  method, Eniter, dbg, belief, nruns] = ...
inputs(varargin);


%maxcomp = d;                          % limit on the number of components
nsplit  = g;                           % split all factors in No. of steps
nsub    = k;                           % No of iteration without commerge
% ----------------------------------------------------------
% --- meaning of variables ---
% Mix0  : initial mixture converted to type 21
% frg   : checked forgetting rate
% ndat  : checked size of data sample to be processed
% niter : checked number of iterations
%
% --- estimation ---
% method: estimation method for "mixest"
% Eniter: number of iterations for "mixest"
% nruns : number of runs for mixture estimation
%
% --- structure estimation ---
% belief: belief for structure estimation
%
% --- modification of processing ---
% dbg   : debug indicator
% nsplit  (g) : split all factors in No. of steps
% nsub    (k) : number of iteration without merging
% c           : final housekeeping is skipped
% ----------------------------------------------------------
% get arguments for buffered processing
% ----------------------------------------------------------
buffarg(ndat);
dbgdf = 0;
% ----------------------------------------------------------
% DEBUG is temporarily reset to 1 and its value is recorded
% it means that iterations in estimation are not displayed
% ----------------------------------------------------------
dbg_memory = dbg;
if dbg>1, DEBUG = 1; end

% ----------------------------------------------------------
% initial mixture estimation
% ----------------------------------------------------------
[s,s0] = mixdfms(Mix0);
if method(1)=='m',
   Mix  = mixestmt(Mix0, frg, ndat, 1);
   Mix0 = mixflat(Mix);
   Mix  = mixestqb(Mix0, frg, ndat, Eniter-1);
else
   Mix  = mixest(Mix0, frg, ndat, Eniter, method);
end

if dbg
   disp('=== mixture initalization ===');
   fprintf('--- initial v-likelihood = %g --- \n',Mix.states.mixll);
   iterplot(Mix0, Mix, 0, ndat);
end

% ----------------------------------------------------------
% set change flag and iteration counter
% ----------------------------------------------------------
   iter = 1;
% ==========================================================
end % --- end of processing without breakpoint ---

% ==========================================================
if length(BREAKPOINT)==0 | ~ischar(BREAKPOINT), breakpoint=0; brkname=[];
else   breakpoint=1; breakname = BREAKPOINT;
end

% ----------------------------------------------------------
% the best result is recorded in the mixture Mixbest
% ----------------------------------------------------------
Mixbest  = Mix;                            % best intermediate results
Mixbest0 = Mix0;

% ==========================================================
for iter = iter:niter  % --- iterations ---
% ==========================================================
    if dbg, fprintf('======= ITERATION %i =======\n',iter); end

tic
%>beginning of each mixinit iteration
% ----------------------------------------------------------
% estimated mixture is flattened
% ----------------------------------------------------------
Mix0 = mixflat(Mix);                      % mixture flattening

% ----------------------------------------------------------
% By mixestee, mixture Mix is estimated by single pass of mixestim.
% During the estimation, a pair of two-component static mixtures is
% also fitted to filtration errors of each factor. The 1st one "Mixs"
% has different components, the 2nd one "Mixs1" has equal components.
% They serve for recognizing whether the factor consists just of
% single "hill" or whether it covers several hills.
% ----------------------------------------------------------
[Mix, Mixs, Mixs1] = mixestee(Mix0, 1, ndat);
if dbgdf, disp('>1<<<<mixestee'), prtdf(Mix0); prtdf(Mix); end

% ----------------------------------------------------------
% v-likelihood of mixtures in  Mixs and Mixs1 is tested.
% The factors "facs" will be split
% ----------------------------------------------------------
if iter<=nsplit,                       % all factors will be split
   facs = Mix.states.facs;
else %~~~~~~~~~~~~~~~~~~~~~~~~~~~~     % the factors suspicious for
                                       % covering more hills will be split
   facs = issplit(Mixs, Mixs1);        % factors to be split
if isempty(facs)                       % processing is finished
   if dbg, disp('MIXINIT - no factors selected'); end
   break;
end
end  %~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if dbg, fprintf('FACTORS: %i of %i are selected for split\n', ...
   length(facs), length(Mixs));
end

% ----------------------------------------------------------
% All components containing a factor selected for splitting are
% by  "mixsplit".
% ----------------------------------------------------------
[Mixx, list] = mixsplit(Mix, Mix0, facs, Mixs, belief, nruns);
if dbgdf, disp('>2<<<<mixsplit, no initial mixture'), prtdf(Mixx); end

% ----------------------------------------------------------
% Split Mix is flattened and estimated to confront it with DATA
% ----------------------------------------------------------
Mix0 = mixflat(Mixx);
if method(1)~='f'
   Mix  = mixestim(Mix0, frg, ndat);
else
   Mix  = mixestim(Mix0,1,ndat);
end

if dbgdf, disp('>4<<<<estimation after flattening'); prtdf(Mix); end

% ----------------------------------------------------------
% Splitting of components may lead to too many components.
% The function commerge  tries to reduce their number.
% ----------------------------------------------------------

if iter>nsub
   [Mix, Mix0] = commerge(Mix, Mix0);
   if length(Mix.dfcs)>5, [Mix, Mix0] = mixerase(Mix, Mix0); end
end

Mix0 = mixflat(Mix);	

if method(1)~='f'
   Mix  = mixestim(Mix0, frg, ndat);
else
   Mix  = mixestim(Mix0,1,ndat);
end



if dbgdf, disp('>5<<<<after merge'); prtdf(Mix0), prtdf(Mix), end

% ----------------------------------------------------------
% Mixture estimation with the fixed structure
% ----------------------------------------------------------
   Mix0 = mixflat(Mix);      % patch MK: merged Mix0 gained by flattening ???
if method(1)=='m',
   Mix  = mixestmt(Mix0, frg, ndat, 1);
   Mix0 = mixflat(Mix);
   Mix  = mixestqb(Mix0, frg, ndat, Eniter-1);
else
   Mix  = mixest(Mix0, frg, ndat, Eniter, method);
end
if dbg, fprintf('MIXINIT: %i components, and mixll=%g has been reached\n', ...
   length(Mix.dfcs), Mix.states.mixll);
end
if dbgdf, disp('>6<<<<after estimation'); prtdf(Mix0); prtdf(Mix); end

% ----------------------------------------------------------
% the "best" result is maintained
% ----------------------------------------------------------
if  Mix.states.mixll>Mixbest.states.mixll
    Mixbest  = Mix;
    Mixbest0 = Mix0;
end

% ----------------------------------------------------------
% show results if DEBUG>0
% ----------------------------------------------------------++
if dbg, iterplot(Mix0, Mix, iter, ndat); end

% BREAKPOINT if any ****************************************
if breakpoint
   clear varargin
   eval(['save ',breakname]);
end

% ==========================================================
end % === end of iterations ===
% ==========================================================

% ----------------------------------------------------------
% get the best result
% ----------------------------------------------------------
if method=='s' % simple editting of mixll
[Mix, Mixbest] = fixmixll(Mix, Mixbest);
if  Mix.states.mixll>Mixbest.states.mixll
    Mixbest  = Mix;
    Mixbest0 = Mix0;
end
end

Mix  = Mixbest;
Mix0 = Mixbest0;

if c % sometimes is better make experiments with final mixture
     % form, by "c" option this is skipped ..................
% ----------------------------------------------------------
% mixture structure estimation
% ----------------------------------------------------------
[Mix, Mix0] = mixstrid(Mix, Mix0, belief, nruns);
if dbgdf, disp('>7<<<<final structure estimation'); prtdf(Mix0); prtdf(Mix); end

% ----------------------------------------------------------
% Cut insignificant components - it will be re-written
% ----------------------------------------------------------
Mix  = mixcut(Mix);

% final estimation
Mix0 = mixflat(Mix);
if method(1)=='m',
   Mix  = mixestmt(Mix0, frg, ndat, 1);
   Mix0 = mixflat(Mix);
   Mix  = mixestqb(Mix0, frg, ndat, Eniter-1);
else
   Mix  = mixest(Mix0, frg, ndat, Eniter, method);
end
if dbgdf, disp('>8<<<<last iteration'); prtdf(Mix0); prtdf(Mix); end
end % ......................................................

% ----------------------------------------------------------
% display result
% ----------------------------------------------------------

if dbg, iterplot(Mix0, Mix, iter, ndat);
   title('result');
end

if dbg
   disp('==========================================================');
   fprintf('MIXINIT finished ncom = %i mixll = %g iteration = %i\n',...
           size(Mix.coms,1), Mix.states.mixll, iter);
   disp('===========================================================');
end

DEBUG = dbg_memory;
% end of mixinit -------------------------------------
% ###buffarg####################################
function buffarg(ndat)
% arguments for batch processing
% this function is not described - it is used
% for buffered estimation, see C-codes

global DATA NDAT 

   if isnumeric(ndat),
      NDAT = ndat;
   else
      if ~iscell(ndat)
         error('invalid "ndat" argument');
      end
      filename = ndat{1};
      mdat     = ndat{2};
      file     = fopen(filename, 'rb');  % open file
      if (~file) error('file not found'); end
      fseek(file, 0, 'eof');
      position = ftell(file);
      NDAT     = position/8/mdat;
      fclose(file);
   end
%   maxerr = sqrt(1 - (0.1/NDAT)^(1/NDAT) );  % ->defaults
% ###facsplit####################################
function [Fac1, Fac2, al ] = ...
              facsplit(Mix, Mix0, fac, Mixs, belief, nruns)
% split factor
% [Fac1, Fac2] = facsplit(Mix, Mix0, fac, Mixs, belief, nruns)
%
% Fac1, Fac2: split factors
% al    : probabilistic weight assigned to Fac1 and Fac2
% Mix   : estimated mixture
% Mix0  : initial mixture
% fac   : factor to be split
% Mixs  : cell array of 2-component mixtures fitted to filtration errors
% belief: cell vector of beliefs for individual channels
% nruns : number of runs for factor structure estimation
%
% Algorithm:
% - factor structure is estimated
% - the significant part is split in a pair of L'DL decompositions
%   that are extended on the original (richer) structure
%
% Design  : P. Nedoma
% Updated : June 2002
% Project : ProDaCTools
% See also: mixinit, facmerge

% Reference:
% Updated  : September 2002, MK
% Problems :


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Fac : factor to be split
% str : its structure
% Fac1: factor with estimated structure
% strMAP: its structure
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Fac  = Mix.Facs{fac};                  % factor to be split
Fac0 = Mix0.Facs{fac};                 % initial factor
str  = Fac.str;                        % factor structure

% structure estimate
belief1 = gbelief(Mix, fac, belief);   % get belief for channel Fac.ychn
changed = 0;                           % is MAP structure shorter ?
if size(str,2)>=1                      % no action for empty structure
   strMAP = facstr(Fac,Fac0,belief1,nruns); % structure estimation
   if streq(str, strMAP)<=0            % MAP structure shorter
      changed  = 1;
      Fac1     = facarx(Fac.ychn, strMAP);  % build new factor
      Fac1.LD  = ld2ld(Fac.LD,str, strMAP); % get marginal from old factor
      Fac1.dfm = Fac.dfm;
   end
end
if ~changed, Fac1 = Fac; end

[Eth, Cth, cove, dfm] = fac2ls(Fac1);  % convert to LS fields

% get auxiliary 2 component mixture
Mix2    = Mixs{fac};
Mix2    = mix2mix(Mix2, 22);           % convert to LS
Fac1s   = Mix2.Facs{1};                % get factors
Fac2s   = Mix2.Facs{2};                %   "
al      = Mix2.dfcs/sum(Mix2.dfcs);    % weights

% shift factors %%%%%%%%%%%%%%%%%%%%% provizorni uprava
if isempty(Cth), Fac1=[]; Fac2 =[]; al=0; return; end
[eve,d] = maxeigv(ld2v(Cth));          % direction of shift
d       = d*Fac.dfm;                   % weighted eigenvalue
cove1   = Fac1s.cove;                  % covariance
cove2   = Fac2s.cove;                  % covariance

% patched, can be negative
arg = d*( cove/(al(1)*(1-al(1)) ) - cove1/(1-al(1)) - cove2/al(1)  );

if arg<0, Fac1=[]; Fac2 =[]; al=0; return; end
dEth2 = sqrt(d);
%dEth2   = sqrt(d*( cove/(al(1)*(1-al(1)) ) ...
%            - cove1/(1-al(1)) - cove2/al(1)  ));

Eth1    = Eth+al(1)*(dEth2*eve)';      % expectation, MK patch
Eth2    = Eth-(1-al(1))*(dEth2 * eve)';% expectation, MK patch

dfm1    = al(1)*dfm;
dfm2    = (1-al(1))*dfm;

Cth1 =  Cth - diag(diag(Cth)) + diag(diag(Cth)/al(1));
Cth2 =  Cth - diag(diag(Cth)) + diag(diag(Cth)/(1-al(1)));

LD1     = ls2ld(Eth1, Cth1, cove1, dfm1);% conversion to L'DL decomposition
LD2     = ls2ld(Eth2, Cth2, cove2, dfm2);% conversion to L'DL decomposition

if changed                               % merge LD1, LD2 into original LD
   str1 = [ [Fac.ychn; 0], str ];
   str2 = [ [Fac.ychn; 0], strMAP ];
   LD1  = ldchng(Fac.LD, str1, LD1, str2);
   LD2  = ldchng(Fac.LD, str1, LD2, str2);
end

% build the split factors
Fac1     = Fac;
Fac1.LD  = LD1;
Fac1.dfm = dfm1;                        % shortened original dfm
Fac2     = Fac;
Fac2.LD  = LD2;
Fac2.dfm = dfm2;                        % shortened original dfm
% ###gbelief####################################
function belief = gbelief(Mix, fac, chbelief)
% generate belief vector for separated structure and belief vectors
%
%            from the list of channels described in chnbelief
% belief = gbelief(Mix, fac, chbelief)
%
% belief  : belief vector qualifying regression structure; see the guide
% Mix     : the mixture where factor in question is
% fac:    : number of the factor to which belief should be generated
% chbelief: cell-vector; one cell defines belief that the channel
%           significantly influence the factor output
%
% Design  : Ladislav Jirsa
% Updated : 15.4.2001
% Project : ProDaCTool
% See also:
%
% Reference:
% Updated  : September 2002, MK
% Problems : just formal changes made; no real check

belief = [];

if isempty(chbelief)
  return;
end

% get some dimensions
nchn  = size(Mix.Facs{fac}.str,2);% number of channels of the given factor
mchn  = size(Mix.coms,2);         % number of modeled channels

% locate fac in Mix.coms
found = 0;
for mchni=1:mchn                  % scan Mix.coms for factor number fac
  if find (Mix.coms(:,mchni) == fac)
    found = 1;
    break;
  end
end

% quit if not found
if found == 0                   % fac not found in Mix.coms
  error('required factor not found in gbelief')   % patch MK
end

% create vector belief
belief = zeros(1,nchn);

% scan Mix.Facs{fac}.str, look for structure items in chbelief{mchni}
lchb  = size (chbelief{mchni},2); % length of chbelief{mchni}
for i=1:nchn
  chbi=0;
  for j=1:lchb
    if chbelief{mchni}(1,j) == Mix.Facs{fac}.str(1,i) & ...   % channel
       chbelief{mchni}(2,j) == Mix.Facs{fac}.str(2,i)         % delay
        chbi = j;
        break;
    end
  end
  if chbi>0   % item found
    belief (i) = chbelief{mchni}(3,chbi);  % values 1, 2, 3 or 4
  else
    belief (i) = 2;                   % 2 = 'probably present' as default
  end
end
% ###mixerase####################################
function [Mix, Mix0] = mixerase(Mix, Mix0)
% cancel components according to increase of v-log-likelihood
% [Mix, Mix0] = initcut(Mix, Mix0)
%
% Mix  : updated estimated mixture
% Mix0 : updated initial mixture
%
% Design : P. Nedoma
% Updated: October 2002
% Project: ProDaCTools

global DEBUG
if length(DEBUG)>0, dbg = DEBUG; else, dbg=0; end
if dbg, fprintf('MIXCUT: no of components %i -> ', length(Mix.dfcs)); end

if nargin==1, Mix0 = mixflat(Mix); end
[ncom, nchn] = size(Mix.coms);

dfcs   = Mix.dfcs;
dfcs0  = Mix0.dfcs;
delete = [];

st = 0; s0 = 0;
for com = 1:ncom
    st      = st + dfcs(com);
    s0      = s0 + dfcs0(com);
    lc(com) = gammaln(dfcs0(com)) - gammaln(dfcs(com));

    for i = 1:nchn
        fac     = Mix.coms(com,i);
        Fac     = Mix.Facs{fac};
        Fac0    = Mix0.Facs{fac};
        lc(com) = lc(com) - loglik(Fac.LD, Fac.dfm, Fac0.LD, Fac0.dfm);%... % MK new 
%        +0.5*(Fac0.LD(1,1)- Fac.LD(1,1) ... 
%        +Fac0.LD(2:end,1)'*diag(diag(Fac0.LD(2:end,2:end)))*Fac0.LD(2:end,1) ...
%         -Fac.LD(2:end,1)'*diag(diag( Fac.LD(2:end,2:end)))* Fac.LD(2:end,1));
    end
end
nc=ncom;                               %MK
for com = 1:ncom
    if nc<2
        break
    end    
    kapt=dfcs(com);
    kap0=dfcs0(com);
    l = lc(com) - gammaln(st-kapt) + gammaln(s0-kap0) ...
        -(st-s0)*log(nc)+(st-s0-kapt+kap0)*log(nc-1) ...
        +gammaln(st)-gammaln(s0); 
    if l>0, delete = [delete, com];
       nc=nc-1;                        %MK    
       st = st - dfcs(com);
       s0 = s0 - dfcs0(com);
    end
end
if isempty(delete),
   if dbg, fprintf('... no is cut\n'); end
   return;
end

while 1
    com = delete(1);
    if length(Mix.dfcs)==1, break; end
    Mix = comdel(Mix, com);
    delete = delete(2:end);
    if isempty(delete), break; end
    delete = delete - 1;
end

if dbg, fprintf('%i remains\n', length(Mix.dfcs)); end
if nargout>1, Mix0 = mixflat(Mix); end
% ###inputs####################################
function [Mix0, frg, ndat, niter, ...
 a, c, d, g, k,...
 method, Eniter, dbg, belief, nruns] = inputs(varargin)
% auxiliary function for mixinit: decomposes inputs, sets defaults
% [Mix,Mix0] = mixinit(Mix0, frg, ndat, niter, opt, belief);

% constants (to be moved to defaults)
defs = defaults('i');
%   [niter, nruns, nrpt, beta, k, g];
%     1      2      3     4     5    6
niter   = 5; % defs(1); needs to be changed in defaults
nruns   = defs(2);                    % number of runs for factor structure estimation
Eniter  = defs(3);                    % number of iterations for iterative est
k       = defs(5);
g       = defs(6);

frgd    = defaults('frg');            % default forgetting rate

inp = varargin{1};
%inp = varargin; % for batch
ni  = length(inp);
% debug run ?
global DEBUG DATA
if length(DEBUG)>0, dbg = DEBUG; else, dbg=0; end

% defaults
if ni<6,  belief = [];   else, belief  = inp{6};   end
if ni<5,  options = '0'; else, options = inp{5};   end
if ni>=4, niter   = inp{4};   end
if ni<3,  ndat    = size(DATA,2);
   if ndat==0, error('global DATA not present'); end
else 
   if isnumeric(inp{3}), ndat = 1 * inp{3};
   else ndat = inp{3};
   end     
end

%[m,n] = size(ndat);
%   if m*n> 1, DATA = ndat; ndat=length(DATA); end
frgd = defaults('frg');
if ni<2, frg = frgd; else, frg = inp{2}; end

Mix0 = inp{1};
Mix0 = mix2mix(Mix0, 21);

% options
c=1; d=0;
if ~isempty(find(options=='c')), c=0; end

%  specification recursive mixture estimation
   i  = find(options=='q' | options=='b' | options=='f' | ...
            options =='m' | options =='p' | options=='Q' | options=='P'); % <<<<<<<<<<<<<<<<<<<<<<<
   if length(i)>1,
      error('only one estimation method (q,b,f,m) must be specified');
   end

   if  ~isempty(i),
       method = options(i);
   else % use mixestim
       Eniter = 1;
       method = 'q';
   end

   i = find(options=='s');
   if ~isempty(i) | method=='b', method = char([method, 's']);
   end

if 0
   i = find(options=='s');
   if ~isempty(i) | method=='m', method = char([method, 's']);
   end
end
% decoding 'a', 'n' and 'h'
   len = length(options);
   i = find(options=='a');
   if isempty(i), a = 7;
   else
      i = i(1) + 1;
      if options(i)<'0' | options(i)>'9'
         error('option ''a'' must be followed by a number');
      end
      n = 0;
      while i<=len & options(i)>='0' & options(i)<='9'
         n = n*10 + double(options(i)-'0');
         i = i + 1;
      end
      a = n;
   end

% number of components
   len = length(options);
   i = find(options=='d');
   if isempty(i), d = 6; % <<<<<<<<<<<<<<<<<< do defaults
   else
      i = i(1) + 1;
      if options(i)<'0' | options(i)>'9'
         error('option ''a'' must be followed by a number');
      end
      n = 0;
      while i<=len & options(i)>='0' & options(i)<='9'
         n = n*10 + double(options(i)-'0');
         i = i + 1;
      end
      d = n;
   end


   len = length(options);
   i = find(options=='n');
   if ~isempty(i)
      i = i(1) + 1;
      if options(i)<'0' | options(i)>'9'
         error('option ''n'' must be followed by a number');
      end
      n = 0;
      while i<=len & options(i)>='0' & options(i)<='9'
         n = n*10 + double(options(i)-'0');
         i = i + 1;
      end
      Eniter = n;
   end

   i   = find(options=='h');
   if ~isempty(i)
      i = i(1) + 1;
      if options(i)<'0' | options(i)>'9'
         error('option ''h'' must be followed by a number');
      end
      n = 0;
      while i<=len & options(i)>='0' & options(i)<='9'
         n = n*10 + double(options(i)-'0');
         i = i + 1;
      end
   end

% decoding k 
   len = length(options);
   i = find(options=='k');
   if isempty(i), k = 2 ;
   else
      i1 = i(1) + 1; i = i1;
      n = 0;
      while i<=len & ...
         ( (options(i)>='0' & options(i)<='9') | options(i)=='.')
         i = i + 1;
      end
      if i>len, i=len;
      elseif i<len, i=i-1;
      end
      eval(['k     = ', options(i1:i), ';']);
   end

% decoding g (2nd optional constant)
   len = length(options);
   i   = find(options=='g');
   if isempty(i), g = 2; % <<<<<<<<<<<<<<< default
   else
      i1 = i(1) + 1; i = i1;
      n = 0;
      while i<=len & ...
         ( (options(i)>='0' & options(i)<='9') | options(i)=='.')
         i = i + 1;
      end
      if     i>len,  i=len;
      elseif i<=len, i=i-1;
      end
      eval(['g = ', options(i1:i), ';']);
   end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ###issplit####################################
function facs = issplit(Mixs, Mixs1)
% select factors to be split - auxiliary function for mixinit
% facs = issplit(Mixs, Mixs1)
%
% facs : candidates for split
% Mixs : two-component mixture on filtration errors;
%        output of mixestee
% Mixs1: one-component mixture on filtration errors; 
%        outputs of mixestee
%
% Design : M. Karny
% Updated: March 2002
% Project: ProDaCTools
% See also: mixinit, mixestee

% Reference:
% Updated  : July 02, MK

nfac = length(Mixs);
vlld = zeros(1,nfac);
for fac=1:nfac
   vll1 = Mixs1{fac}.states.mixll;  % one-component
   vll  = Mixs{fac}.states.mixll;   % two-components
   vlld(fac) = vll-vll1;
end

[vlld, facs] = sort(vlld);
ii           = find(vlld>0);
if isempty(ii), facs=[];
else,           facs=facs(ii);
                facs = facs(length(facs):-1:1);
end
% ###maxeigv####################################
function [eve,d] = maxeigv(Cth);
%  maximum eigenvector eve and eigenvalue d of the matrix Cth>0
%  eve = maxeigv(Cth)
%
% eve  : eigenvector                              column vector
% d    : eigenvalue 
% Cth  : covariance matrix (symmetrix positive semi-definite)
%
% Designed: M. Karny
% Updated : 10.09.00
% Project : ProDaCTools
% See also: facsplit
%
% Reference:
% Updated  : July 02, MK
% Problems : defaults should be taken out, mex-version
%            should be created

% defaults
eps   = 1e-6;                 % zero level
Niter = 500;                  % upper bound on the number of iterations
nPsi  = length(Cth);
eve = ones(nPsi,1)/sqrt(nPsi);
tr  = trace(Cth);
Cth = Cth / tr;

% iterations
dist = 1e60;                  % distance of subsequent guesses

for iter=1:Niter
    if dist<eps
       break
    end   
    dist=0;
    nor=0;                              % norm;
    for i=1:nPsi                        % product
        s=0; 
	for k=1:nPsi
	    s=s+Cth(i,k)*eve(k,1);
	end
	evep(i,1)=s;
	nor=nor+s*s;
    end
    nor=sqrt(nor);
    for i=1:nPsi
        au=eve(i,1);
	eve(i,1)=evep(i,1)/nor;
	au=au-eve(i,1);
	dist=dist+au*au;
    end
    dist=sqrt(dist);
end
d=nor * tr;
% ###mixestee####################################
function [Mix, Mixs, Mixs1] = mixestee(Mix0, frg, ndat)
% quasi-Bayes mixture estimation fitting two-component mixtures
% to filtration errors of all factors
% [Mix, Mixs, Mixs1] = mixestee(Mix0, frg, ndat)
% [Mix, Mixs, Mixs1] = mixestee(Mix0, frg)       ndat = sample size
% [Mix, Mixs, Mixs1] = mixestee(Mix0)            frg = defaults('frg');
%
% Mix  : estimated mixture
% Mixs : cell vector of mixtures fitted to filtration errors,
%        each is static, having 2 different components
% Mixs1: cell vector of mixtures fitted to filtration errors,
%        each is static, having 2 identical components
% Mix0 : initial mixture
% frg  : forgetting rate
% ndat : size of data sample
%
% Design  : P. Nedoma
% Updated : May 2002
% Project : ProDaCTools
% See also: mixinit, issplit, facsplit

% Reference:
% Updated  : 28.09.02, MK
% Problems : non-stadard use of global variables; defaults should go out

global DATA NDAT

if Mix0.type>100, error('mixestee is designed for estimators only'); end
Mix0 = mix2mix(Mix0,21);                            % must be of 21 type

if nargin < 3, ndat = []; end
if nargin < 2, frg  = []; end

if isempty(frg),
   frg = defaults('frg');
end
if isempty(ndat),
   ndat = size(DATA,2);
end

% building mixtures for estimation of mixtures on  filtering errors
%  1,5
eprange  = 5;                         % DATA are normalized
DFM      = 10;                        % MK defaults should go out
%%%% !!!!!!!!!!!!!!!!!!! do defaults
if iscell(ndat)
     n = NDAT;                        % for buffered estimation
else n = ndat;
end

nfac = length(Mix0.states.facs);      % number of cells needed

% this is fixed acording to experience
Mixf = genmixe(2, 1, [0;1], n);%, 1, 0.0001, 10, [100 100]); % MK sensitive to external
Mixf.Coms{1}.Eth =  eprange;          % hidden changes: it should have
Mixf.Coms{1}.dfm =  DFM;              % private defaults

Mixf.Coms{2}.Eth = -eprange;
Mixf.Coms{2}.dfm =  DFM;

Mixf = mix2mix(Mixf,21);

Mixs = cell(1, nfac);
for i=1:nfac,
    Mixs{i} = Mixf;
end

% array of identical components
% this is fixed acording to experience
Mixf = genmixe(2, 1, [0;1], n);%, 1, 0.0001, 10, [100, 100]); % MK sensitive to external

   Mixf.Coms{1}.Eth = 0;
   Mixf.Coms{1}.dfm = DFM;
   Mixf.Coms{2}.Eth = 0;
   Mixf.Coms{2}.dfm = DFM;

   Mixf  = mix2mix(Mixf,21);
   Mixs1 = cell(1, nfac);
   for i = 1:nfac,
       Mixs1{i} = Mixf;
   end

% mixestfe realized as MEX
[Mix, Mixs, Mixs1] = mixestfe(Mix0, frg, ndat, Mixs, Mixs1);
% ###mixfix####################################
function Mix1 = mixfix(Mix)
% reorganize mixture after deleting of a component
% Mix = mixfix(Mix)
%
% Mix: updated mixture
%
% Design : P. Nedoma
% Updated: January 2001
% Project: ProDaCTools

nfac  = length(Mix.Facs);
flist = zeros(1,nfac);
coms  = Mix.coms;
[ncom, nchn] = size(coms);
ifac = 1;
coms1 = [];
for com = 1:ncom
    facs  = coms(com,:);
    facs1 = 0*facs;

    for i=1:nchn, fac = facs(i);
        j = flist(fac);

        if j==0
           facs1(i)   = ifac;
           flist(fac) = ifac;
           ifac = ifac+1;
        else
           facs1(i) = j;
         end

    end
    coms1 = [coms1; facs1];
end
nfac = ifac-1;
Facs = cell(1, nfac);
dfm0 = [];
facwgs = [];
faceps = [];
for i=1:nfac, j = find(i==flist); Facs{i} = Mix.Facs{j};
    dfm0(i)   = Mix.states.dfm0(j);
    facwgs(i) = Mix.states.facwgs(j);
    faceps(i) = Mix.states.faceps(j);
end
Mix1 = mixconst(Facs, coms1, Mix.dfcs);

Mix1.states.dfm0   = dfm0;
Mix1.states.dfcs0  = Mix.states.dfcs0;
Mix1.states.comlls = Mix.states.comlls;
Mix1.states.mixll  = Mix.states.mixll;
Mix1.states.comwgs = Mix.states.comwgs;
Mix1.states.facwgs = facwgs;
Mix1.states.faceps = faceps;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ###mixsplit####################################
function [Mix1, list]  = mixsplit(Mix, Mix0, facs, Mixs, belief, nruns)
% split mixture
% [Mix1,list] = mixsplit(Mix, Mix0, facs, Mixs, belief, nruns)
%
% Mix1  : split mixture
% list  : list of components
%          row 1 changed component (of Mix)
%          row 2 added component
% Mix   : estimated to be split mixture
% Mix0  : initial mixture
% facs  : factors to be split
% Mixs  : cell array of 2-components mixtures fitted to filtration errors
% belief: cell vector of beliefs for individual channels
% nruns : number of iterations for iterative estimation
%
% Design  : P. Nedoma
% Updated : March 2002
% Project : ProDaCTools
% See also: facsplit, mixinit

% Reference:
% Updated  : 28.09.02 MK
% Problems: substantial changes with respect to df
%           check is necessary; it would be good to return back
%           possibility to deal with common factors

global DEBUG
if length(DEBUG)>0, dbg = DEBUG; else, dbg=0; end

% tests of arguments
if Mix.type~=21,
   error('mixsplit is designed for ARX mixture only');
end

% dimensions
[ncom,nchn] = size(Mix.coms);          % dimensions
nfac  = length(facs);                  % number of factors to be split
ncom0 = ncom;                          % initial number of components

% components to be split
coms0 = [];
for i = 1:nfac
    fac  = facs(i);
    coms0 = [coms0, Mix.states.mapping{fac}];
end
coms0 = unique(coms0);

if dbg, fprintf('MIXSPLIT %i components from %i will be split\n', ...
     length(coms0), length(Mix.dfcs) );
end

% for an easy access
Facs  = Mix.Facs;
coms  = Mix.coms;
dfm0  = Mix.states.dfm0;
dfcs0 = Mix.states.dfcs0;
dfcs  = Mix.dfcs;
n     = length(Facs) + 1;

% !!!! spolecne factory
% the trick with FacsSplit leads to common factors
%FacsSplit = zeros(1, n);         % MK this option should be returned back
list = [];
%!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
DFM0 = sum(dfm0)/length(dfcs);         % this is maintained

for com = coms0   % ============== over components =========
    facs0 = Mix.coms(com,:);           % factors in component
    facs1 = facs0;                     % new factors in component
    ala= 0;                            % MK patch: average factor weight
    for i = 1:nchn                     % cycle over component factors
        fac = facs0(i);                % factor

%        if FacsSplit(fac)             %MK: return back: the factor already split
%           facs1(i) = FacsSplit(fac);
%        else
%df=Mix.Facs{fac}.dfm
                                       % MK patch corresponding
                                       % to the changed facsplit
   j  = find(fac==facs);
   if ~isempty(j)
      [Fac1, Fac2,al] = facsplit(Mix, Mix0, fac, Mixs, belief, nruns);
      if isempty(Fac1), j = []; end
   end
%patch: cove<cove1+cove2
   if isempty(j)                       % make factor copy
      Fac1 = Mix.Facs{fac};            % get factor
      LD   = Fac1.LD;
      Fac1.LD = LD*0.999;   
      Fac1.dfm = 0.5*Fac1.dfm;         % dfm
      Fac2 = Fac1;   
      Fac2.LD  = 0.99999*LD;                     % factor copy
      al=[0.5,0.5];                    % weights
   end

   if randn<0,   % ---------------------
      Facs{fac} = Fac1;                % replace original factor
      Facs{n}   = Fac2;                % add new factor
      alpha     = al(1);               % weight: MK patch
   else
      Facs{n}   = Fac1;                % replace original factor
      Facs{fac} = Fac2;                % add new factor
      alpha     = 1-al(1);             % weight: MK patch
   end           % ---------------------

   facs1(i)  = n;                 % edit new component

   df = dfm0(fac);
   dfm0(fac) = alpha* df;              % MK patch
   dfm0(n)   = (1-alpha)*df;           % MK patch extend dfm0
            ala = ala + alpha;              % MK patch: sum of alpha
%           dfm0(n)   = dfm0(fac);        % MK patch extend dfm0
%           FacsSplit(fac) = n;
           n         = n + 1;
%        end
    end              % ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   ala=ala/nchn;                            % MK normalize
  coms       = [coms; facs1];               % new components
% rozsirena
  dfcs       = [dfcs,(1-ala)*dfcs(com)];    % MK patch begin: extended dfcs
  dfcs0      = [dfcs0,(1-ala)*dfcs0(com)];  % extended dfcs0
  dfcs(com)  = ala*dfcs(com);               % correct dfcs(com)
  dfcs0(com) = ala*dfcs0(com);              % correct dfcs0(com)
%    dfcs1      = dfcs(com)/2;              % dfcs of component is split
%    dfcs       = [dfcs, dfcs1];            % patch extended dfcs
%    dfcs(com)  = dfcs1;                    % split dfcs com-component
%    dfcs1      = dfcs0(com)/2;             % the same for dfcs0
%    dfcs0(com) = dfcs1;                    % MK patch end

    list = [list, [com; size(coms,1)] ];
end  % === over components =================================


% corrected dfm0 - MK 4.7                   % MK whole block removed
%if 0                                       % DM0 and DFM1 not needed
%DFM1 = sum(dfm0)/length(dfcs);
%k    = DFM0/DFM1;
%dfm0 = k*dfm0;
%end

%if 1 % <<<<<<<<<<<<<<<<<<<<<<<< experiments should be made
% correction of dfm0
%ncom     = length(dfcs);
%sumdfm0  = sum(dfm0);
%ddfm     = (sumdfm0 - ncom*DFM0);      % correction of ddfm
%wgs      =  dfcs0/sum(dfcs0);          % weights
%ddcom    =  ddfm * wgs;                % increment of components
%for com=1:ncom
%    facs   = coms(com,:);
%    dfmnew = (dfm0(facs(1)) - ddcom(com))/nchn;
%    for  fac=facs
%         dfm0(fac) = dfmnew;
%    end
%end
%end

% build output mixture
  Mix1 = mixconst(Facs, coms, dfcs);

% preserve initial states
  Mix1.states.dfcs0 = dfcs0;
  Mix1.states.dfm0  = dfm0;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [Mix, Mix1] = fixmill(Mix, Mix1)
%
%  Mix: reference Mixtuand Mix1 has the same starting conditions
%  Mix1 is processed with the stopping rules

% processing time
s  = sum (Mix.dfcs  - Mix.states.dfcs0);    
s1 = sum (Mix1.dfcs - Mix1.states.dfcs0); 


flag = 0;
if s1<s, flag=1; s0=s; s=s1; s1=s0;  Mixx=Mix; Mix=Mix1; Mix1=Mixx; end

d   = Mix.states.mixll/s;                % for one step
vll = Mix.states.mixll + (s1-s)*d;        
df  = Mix.dfcs;
wgs = df/sum(df);
dfcs = df + (s1-s)*wgs;

Mix.states.mixll = vll;
Mix.dfcs = dfcs;



if flag, Mixx=Mix; Mix=Mix1; Mix1=Mixx; end

