function [res, tstop] = simeval(Sim, chns, nrep, ndat, threshold)
%  repeated simulations
%  [res, tstop] = simeval(Sim, chns, nrep, ndat, threshold)
%  [res, tstop] = simeval(Sim, chns, nrep, ndat) threshold = 0.01
%  [res, tstop] = simeval(Sim, chns, nrep) size(DATA,2)
%  [res, tstop] = simeval(Sim, chns) 100
%  [res, tstop] = simeval(Sim) all modelled channels
%
%  res   : processing results: cell vector for each channel of chns
%          res{chn}.stats - confidence interval for range and increments
%          res{chn}.tra   - trajectories of individual simulation runs
%  tstop : time where trajectories are stopped
%  Sim   : simulator | task
%  chns  : list of channels to be evaluated
%  nrep  : maximum number of repetitions
%  ndat  : maximum length of trajectories
%  threshold : threshold value, default = 0.01;

% See also: mixsimul, credit, stopt
% References: M. Karny: When Estimation Reaches Steady State ?

% Design : P. Nedoma
% Updated: May 2004
% Project: GA CR 102/03/0049, AV CR S1075351, S1075102, DESIGNER

%!!!!!!!!!!!!!!!!!!!!!!!!!!! 2*threshodl used for number of repetions 
dbg = 0;
x=defaults('E'); beta = x(3);              % default credibility

global DATA TIME
if ~nargin, error('use: [res,tstop]=simeval(Sim,chns,nrep,ndat,threshold)'); end
if nargin<4 | isempty(ndat), ndat  = size(DATA,2); end
if nargin<5,  threshold = x(2); end
                            
if nargin<3 | isempty(nrep), nrep  = 100;                    end
if nargin<2 | isempty(chns), chns = Sim.states.modelled;     end

% simulation version
if     isstruct(Sim), sim = 1;
elseif iscell(Sim),   sim = 2;
else   error('1st argument is simulation program');
end

% time range
if      length(ndat)==2, tmin=ndat(1); tmax=ndat(2);
elseif  sim==1, tmin = mixmaxtd(Sim)+1;  tmax=ndat;
else    tmin = 1; tmax=ndat;
end
ndat = tmax;

nchn  = length(chns);                  % No. of outputs

% tests of arguments
if ndat>size(DATA,2), error('DATA not sufficiently dimensioned'); end

% auxiliary matrices
nrep0 = 3*beta/(1-beta) - 1;
%if nrep0>30, nrep0 = 30; end
tra   = zeros(nrep, ndat);             % recorded trajectories
mean1 = zeros(1, nrep);                % sorted trajectories of means
tstop = ndat+zeros(1,nrep);            % times when trajectory stopped

% form processing stuctures
if isnumeric(chns)
   for i = 1:nchn
       ychn = chns(i);
       str  = [ychn; 1];               % 1st order
       Facs0{i}  = facarx(ychn, str);
   end
else Facs0 = chns;
end

for i = 1:nchn
   res{i}   = tra;
   means{i} = mean1;
end

% simulation
for rep=1:nrep %============================================
fprintf('.'); if ~rem(rep,10), fprintf('\n'); end
   Facs = Facs0;

for TIME = tmin:tmax % -------------------------------------
   switch sim
      case 1,   Sim=mixsimul(Sim);
      case 2,   Sim{2} = feval(Sim{1}, Sim{2});
   end
% here you can include your program

Q = 0;
for i = 1:nchn % .............................................
   Fac = Facs{i};
   y   = getdvect([Fac.ychn; 0]);
   res{i}(rep, TIME) = y;
   [Fac,q] = stopstac(Fac);
   Facs{i} = Fac;
   Q = Q + q;
end % ......................................................
   if Q<threshold, break; end
end % ------------------------------------------------------
if dbg, plot(Qs); ax=axis; ax(4)=0.05; axis(ax); drawnow; end
tstop(rep) = TIME;

% compute a run characteristic - here mean
tra1 = res{i}(rep, 1:tstop(rep));
for i = 1:nchn
    if rep==1, means{i}(1) = mean(tra1);
    else
       means{i}(1:rep) = merge(means{i}(1:rep-1),mean(tra1));
    end
end

% stopping for number of repetitions
if rep>nrep0
   for i = 1:nchn
       [Cl,Cu,Chat,flag,Q]=credits(means{i}(1:rep),beta,2*threshold);
       if i==1,  finished = 1; end
       if ~flag, finished = 0; break; end
   end
   if finished, nrep = rep; break; end
end

end % ======================================================
tstop = tstop(1:nrep);
res = eval2(res, tstop, Facs, beta);
% ##########################################################
function result = eval2(res, tstop, Facs, beta)
% compute simulation results
% result = eval2(res, tstop, Facs, beta)
%
% result: structure of results
% res  : recorded trajectories
% tstop : end of trajectories
% Facs  : simulation factors
% beta  : credibility level in (0,1), default 0.7

% Design : P. Nedoma
% Updated: July 2003
% Project: GA CR 102/03/0049, AV CR S1075351, Designer

nchn = length(res);
nrep1 = length(tstop);
ndat1 = max(tstop);

for i=1:nchn
   % make conversion
    resnew = res{i}(1:nrep1, 1:ndat1);
    r = eval1(Facs{i}.ychn, resnew, tstop, beta);

result{i} = struct(...
'stats', r, ...
'tra',    resnew);
end

% ##########################################################
function result = eval1(ychn, tra, tstop, beta)
% compute simulation result
% result = results(tra, tstop, beta)
%
% result: structure of results
% tra   : recorded trajectories
% tstop : end of trajectories
% beta  : credibility level in (0,1), default 0.7

% Design : P. Nedoma
% Updated: July 2003
% Project: GA CR 102/03/0049, AV CR S1075351, Designer

if nargin<4, beta = 0.7; end
[nrep, ndat] = size(tra);
if length(tstop)<nrep,

end
% mean, std, median, ranges
l = zeros(1, nrep); h = l; u = l;
for i=1:nrep
    dat    = tra(i,  1: tstop(i) );
    [l(i), u(i), h(i)] = credit(dat, beta);
end
y  = [l', h', u'];
y  = mean(y);

% increments
for i=2:nrep
    for j=tstop(i):-1:2
        tra(i, j) = tra(i,j) - tra(i, j-1);
    end
end

% input characteristics
for i=1:nrep
    dat    = tra(i,  1: tstop(i) );
    [l(i), u(i), h(i)] = credit(dat, beta);
end
yincr  = [l', h', u'];
yincr  = mean(yincr);

result = struct(...
'chn', ychn, 'beta', beta, ...
'range', y, 'incr', yincr);
% ##########################################################
function [Cl,Cu,Chat,flag,st]=credits1(C,beta,c)
% evaluation of an credibility interval with stopping
%
% [Cl,Cu,Chat,flag,st] = credits(C,beta,c)
%
% Cl   = lower credibility bound
% Cu   = upper credibility bound
% Chat = center of the credibility interval
% flag = stopping flag [1/0] = stop/do_not_stop data acqusition
% st   = value of the stopping test statistic
% C    = no-vector of independent realizations SORTED
% beta = credibility level in (0,1)
% c    = price of single evaluation

% designed = MK
% updated  = 1.7.03
% project  = RECiAS
% reference= stationary.tex
% see also = test, psi

if nargin<3
   error('credits needs two input  arguments')
end
if beta<=0 | beta>=1
   error('the second argument of credit has to be in (0,1)')
end
no=max(size(C));              % dimension
jo     = floor(no*(1-beta))+1; % the number of points to be omitted

if jo < 3
flag = 0; Cl=[]; Cu=[]; Chat=mean(C); st =[];
return
   error('the number of realizations is too small')
end
yt  = C(no);                  % the newest data item
%C   = sort(C);                % sorted version of data
jl  = 1;
ju  = no;
for j=1:jo;
  if C(jl+1)-C(jl)>C(ju)-C(ju-1)
     jl=jl+1;
  else
     ju=ju-1;
 end
end
Cl  = C(jl);                % lower credibility bound
Cu  = C(ju);                % upper credibility bound
Chat= 0.5*(Cl+Cu);          % center of credibility interval

% stopping test statistics
 nt = no +3;                % degrees of freedom
 if yt<Cl
   Vy =jl+1;
   V1 = ju+1;
   V2 = nt-jo+2;
 elseif yt<=Cu
   Vy = nt-jo+2;
   V1 =jl+1;
   V2 =ju+1;
 else
   Vy = ju +1;
   V1 = jl +1;
   V2 = nt-jo+2;   
 end       
 thetay= Vy/nt;
 theta1= V1/nt;
 theta2=V2/nt;
 st = 2/nt - thetay*log(Vy/((nt+1)*thetay-1))...
            + theta1* (psi(V1)-psi(Vy)-log((nt-1)*theta1/Vy)) ...
            + theta2* (psi(V2)-psi(Vy)-log((nt-1)*theta2/Vy));
  if c>st
     flag = 1;
   else
     flag =0;
  end
% ##########################################################
function v = merge(v, val)
% merge val into sorted v

len=length(v);
if val>v(len), v(len+1)=val;
else
   for i=len:-1:1
      if val<v(i), v(i+1) = v(i);
      else, v(i+1)=val; break;
      end
      if i==1, v(1)=val; end
   end
end
