% case study: model verification
%             dynamic SISO model
%
% Design : P. Nedoma
% Updated: October 2003
% Project: GA CR 102/03/0049, AV CR S1075351, S1075102, DESIGNER

prodini                                % startup routine

echo off
idef = [];

while 1 % ==============================================
if isempty(idef)
threshold  = 0.0025;
covy       = 1e-4;
covu       = 1e-4;
ndat       = 1000;
beta       = 0.7;    
seed       = 12345;
maxdat     = 500;
% for simeval
chns       = 1;                            % list of channels
maxrep     = 500;                          % maximum number of repetitions
end % ==================================================

clc
fprintf('Case study: SISO model characteristics via extended simulation \n');
fprintf('--------------------------------------------------------------- \n');
fprintf('\nSystem: SISO, dynamic factor 2nd order,  open loop\n');
fprintf('factor noise covariance ->   covy   = %g\n', covy);
fprintf('open loop covariance    ->   covu   = %g;\n',covu);

fprintf('\n---  Data for estimation -----------------------\n');
fprintf('size                    ->   ndat   = %i;\n',       ndat);
fprintf('seed                    ->   seed   = %i;\n',       seed);

fprintf('\n--- Stopping rules -----------------------------\n');
fprintf('maximum data length     ->   maxdat     = %i;\n',      maxdat);
fprintf('maximum repetitions     ->   maxrep     = %i;\n',      maxrep);
fprintf('threshold               ->   threshold  = %g;\n',      threshold);
fprintf('credibility level       ->   beta       = %g;\n',      beta);
fprintf('--------------------------------------------------\n');

dial1;

clf; figure(1);
sub        = 220;
LegendSize = 12;

echo off
% ================================

% ==========================================================
ychn = 1;                              % output channel
uchn = 2;                              % input  channel

% simulation factor
str = [1 1 2 2; 1 2 0 1];              % factor structure
Eth = [1.81 -0.8187  0.00468 0.00438]; % regression coeficients
Fac = facarxls(ychn, str);             % output factor
Fac.Eth  = Eth;                        % set regression coefficients
Fac.cove = covy;                       % set noise covariance

% simulator
Sim = mixconst(Fac, 1, 1);             % simulator

% data sample, input is noise
if seed, randn('seed', seed); rand('seed', seed); end
DATA = [ zeros(1, ndat); sqrt(covu)*randn(1,ndat)];  % pre-allocated data sample
mixsimul(Sim, ndat);                   % generate data sample

% ==========================================================
% model, estimation
% ==========================================================
Fac = facarxls(ychn, str);             % output factor

% initial mixture for estimation
Fac.dfm  = 10;                         % degree of freedom
Fac.cove = 0.01;                       % noise covariance
Fac.Cth  = 100*diag(ones(1,4));        % covariace matrix of coefficiens
Mix0     = mixconst(Fac, 1, 1);        % initial mixture

% estimation
Mix      = mixestim(Mix0, 1, ndat);   % batch mixture estimation
% ----------------------------------------------------------
% display of estimation results
Fac = arx2arx( Mix.Facs{1} );          % conversion to ARX LS form
prt('Eth simulated: ', Eth);           % true regression coefficients
prt('Eth estimated: ', Fac.Eth);       % estimated regression coefficients
a=sum(Fac.Eth(1:2));  b=sum(Fac.Eth(3:4));
gain = b/(1-a);                         % gain
fprintf('covy = %g  estimated = %g  gain estimated = %g\n',...
        covy, Fac.cove, gain); 
% ----------------------------------------------------------

% ==========================================================
% model, simulated characteristics
% ==========================================================
% build simulator
Sim = arx2arx(Mix);                     % convert model to simulator

% pre-allocated data, references
t0   = 5;                               % time of step
DATA = [zeros(1, maxdat); [ zeros(1, t0), ones(1,maxdat-t0) ] ];

% run
[res, tstop] = simeval(Sim, chns, maxrep, maxdat, threshold);

% ----------------------------------------------------------
% simulation results
disp(' ');
disp('Simulation results');
echo on
% results ------------------------------------------------
res
res = res{1}
res.stats                              % confidence interval for ranges and incremenst
tra   = res.tra;                       % simulation trajectories

echo off

[r,c] = size(tra);
fprintf('maxdat:  %i ->  %i\n', maxdat, c);
fprintf('maxrep:  %i ->  %i\n', maxrep, r);
nrep = r;


t=tra(1:10:nrep,:);
sub=sub+1; subplot(sub);
plot(t');
H=legend('Trajectories');
set(H, 'fontSize', LegendSize); 
grid on
drawnow

                        % credibility
gs = zeros(1, r);
for i=1:r, gs(i) = tra( i, tstop(i) ); end

% gain confidence interval
[gsl,gsu,gshat] = credit(gs, beta);
gain = [gsl, gshat, gsu]

for i=1:r, ov(i) = max([1,abs(tra( i, 1:tstop(i) ))]); end
[ovl,ovu,ovhat]=credit(ov, beta);
overshoots = [ovl, ovhat, ovu]

% histogram of overshoots
sub=sub+1; subplot(sub);
hist(ov,20); grid on
H = legend('Overshoots');
set(H, 'fontSize', LegendSize); 
colormap([127/255 1 212/255])

drawnow

low  = min(tra);
high = max(tra);
tmin = min(tstop);
low  = low(1:tmin);
high = high(1:tmin);


sub=sub+1; subplot(sub);
plot(low); hold on
plot(high);
grid on
H=legend('Envelope');
set(H, 'fontSize', LegendSize); 



% trajectories
n  = size(tra,1);
s  = tra(1,:);
ss = tra(1,:).*tra(1,:);
for i=2:n
    s  = s + tra(i,:);
    ss = ss + tra(i, :).*tra(i,:);
end

m = s / n;
s = sqrt( (ss - 2*s.*m + n * m.*m)/(n-1));

m = m(1:tmin);
s = s(1:tmin);
sub=sub+1; subplot(sub);
plot(m); hold on
plot(m+2*s,':');
plot(m-2*s,':');
grid on
H=legend('Mean +- 2*std');
set(H, 'fontSize', LegendSize); 
setaxis(223:224);
plt('zsimeval')

dial2;
end % ----------------------------------------------------------

