function [Mix, mixlls] = mixest(Mix0, frg, ndat, niter, method)
% iterative Bayesian ARX mixture estimation
%
% LHS = mixest(Mix0, frg, ndat, niter, method)
% LHS = mixest(Mix0, frg, ndat, niter)           method = quasi-Bayes (q) without stopping
% LHS = mixest(Mix0, frg, ndat)                  niter  = 1 for p, q,f s; 10 for b
% LHS = mixest(Mix0, frg)                        ndat   = lenght(DATA)
% LHS = mixest(Mix0)                             frg    = defaults
%
% LHS    : Mix or [Mix, mixlls] 
% Mix    : estimated ARX mixture,            type = 21
% Mix0   : initial estimate of ARX mixture of any type 
% mixlls : trajectory of mixll, i.e. v-log-likelihood of the mixture
%          (used for experimentation only)
% frg    : forgetting rate (alternative forgetting for method (f))
% ndat   : size of data sample or data sample (copied to global DATA)
% niter  : number of iterations used in iterative methods
% method : character string, the 1st significant character means:
%          'p' : projection    
%          'P' : + stopping     
%          'q' : iterative quasi-Bayes mixture estimation
%          'Q': 'q' with stopping
%          'b' : iterative batch quasi-Bayes mixture estimation
%          'f' : iterative mixture estimation with forgetting branching
%   if the string 'method' 2nd column is 's' : states are
%      evaluated by mixstats
%
% Design  : P. Nedoma
% Updated : May 2004
% Project : GA CR 102/03/0049, AV CR S1075351, S1075102, DESIGNER
% See also: mixestqb,mixestq,mixestbb,mistats
% Note: DEBUG mechanism is exploited for inspecting intermediate results
%       DEBUG is set in Prodini

% Reference:
% Updated  : June 2002, MK 
% Problems : if method='f' & frg is non-trivial it is overwritten by  
%            default: marked by ?????

% debug prints 
 global DEBUG DATA TIME
 if length(DEBUG)==0, dbg=0; else, dbg=DEBUG;end 
% defaults
   if nargin<5, method = 'q'; end    % default method 
   if nargin<4,                      % default number of iterations
      switch method(1)
        case 'q', niter = 1;
        case 'Q', niter = 1;
        case 'p', niter = 1;
        case 'p', niter = 1;
        case 'b', niter = 10;
        case 'f', niter = 1;
        case 'm', niter = 1;
        otherwise, error('invalid method specified');
      end
   end
   if nargin < 3, ndat = size(DATA,2);
      if ndat<1, error('global DATA not specified'); end
   else
      [m,n] = size(ndat);
      if m*n>1, DATA = ndat; ndat = n; end
   end
   if nargin<2
      [frgd, frg]  = defaults('frg');
      if method(1)~='f', frg = frgd; end
   end
   if method(1)=='f',
%%%%% ??????? %%%%%%%%%%
      frgd = defaults('frg');
      if frg>=frgd, error('alternative forgetting must be below default one'); end
   end

   Mix0 = mix2mix(Mix0, 21);         % conversion of Mix0 to type 21

if dbg, fprintf('running estimation, wait a moment ...\n'); end

% interface to estimation functions
   switch method(1)
      case 'q'
        if nargout<2, Mix           = mixestqb(Mix0, frg, ndat, niter);
        else          [Mix, mixlls] = mixestqb(Mix0, frg, ndat, niter);
        end
      case 'Q'
        if nargout<2, Mix           = mixestqbs(Mix0, frg, ndat, niter);
        else          [Mix, mixlls] = mixestqbs(Mix0, frg, ndat, niter); % mixlls -> stop times 
        end
      case 'p'
        if nargout<2, Mix           = mixestpb(Mix0, frg, ndat, niter);
        else          [Mix, mixlls] = mixestpb(Mix0, frg, ndat, niter);
        end
      case 'P'
        if nargout<2, Mix           = mixestpbs(Mix0, frg, ndat, niter);
        else          [Mix, mixlls] = mixestpbs(Mix0, frg, ndat, niter);
        end
      case 'b'
        if nargout<2, Mix           = mixestbq(Mix0, frg, ndat, niter);
        else          [Mix, mixlls] = mixestbq(Mix0, frg, ndat, niter);
        end
      case 'f'
        if nargout<2, Mix           = mixestbb(Mix0, frg, ndat, niter);
        else          [Mix, mixlls] = mixestbb(Mix0, frg, ndat, niter);
        end
      case 'm'
        Mix  = mixestmt(Mix0, frg, ndat, niter);
        mixlls = [];
      otherwise
        error('invalid estimation method');
   end

if (length(method)>1) & method(2)=='s'  % evaluation of statistics
   Mix = mixstats(Mix, ndat);
end
